import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { UsersService } from '../users/users.service';
import { JwtService } from '@nestjs/jwt';
import { LoginDto } from './login-dto';
import { CreateUserDto } from '../users/create-user.dto';
import { DateTime } from 'luxon';
import { User } from '../users/user.schema';

@Injectable()
export class AuthService {
  constructor(private usersService: UsersService, private jwtService: JwtService) {}

  async validateUser(loginDto: LoginDto): Promise<any> {
    const user = await this.usersService.findOne(loginDto.email);
    if (user) {
      return user;
    }
    return null;
  }

  async login(loginDto: LoginDto): Promise<{ username; token }> {
    // find user in db
    const user: User = await this.usersService.findByLogin(loginDto);
    if (!user.emailVerified) {
      throw new HttpException('Invalid credentials', HttpStatus.UNAUTHORIZED);
    }

    // generate and sign token
    const token = this._createToken(user);

    return {
      username: user.email,
      ...token,
    };
  }

  private _createToken({ email }: CreateUserDto): any {
    const local = DateTime.local().setZone('Europe/Paris');
    return {
      accessToken: this.jwtService.sign({ email }),
      expiresAt: local.plus({ days: 1 }),
    };
  }
}
